/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/

package org.apache.cocoon.serialization;

import org.apache.cocoon.caching.CacheValidity;
import org.apache.cocoon.caching.Cacheable;
import org.apache.cocoon.components.CocoonComponentManager;
import org.apache.cocoon.environment.Source;
import org.apache.cocoon.environment.SourceResolver;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * A serializer that builds Zip archives by aggregating several sources.
 * <p>
 * The input document should describe entries of the archive by means of
 * their name (which can be a path) in the archive and the source of the entry
 * contents. These are Cocoon sources, and as such can use any of the protocols
 * handled by Cocoon, including "cocoon:" to include dynamically generated
 * content in the archive.
 * <p>
 * Example :
 * <pre>
 *   &lt;zip:archive xmlns:zip="http://apache.org/cocoon/zip-archive/1.0"&gt;
 *     &lt;zip:entry name="foo.html" src="cocoon://dynFoo.html"/&gt;
 *     &lt;zip:entry name="images/bar.jpeg" src="bar.jpeg"/&gt;
 *   &lt;/zip:archive:zip&gt;
 * </pre>
 * 
 * @author <a href="http://www.apache.org/~sylvain">Sylvain Wallez</a>
 */

// TODO (1) : handle more attributes on <zip> for properties of ZipOutputStream
//            such as comment or default compression method and level

// TODO (2) : handle more attributes on <entry> for properties of ZipEntry 
//            (compression method and level, time, comment, etc.)

// TODO (3) : allow the entry to be inlined by using "serializer" instead of "src", e.g. :
//            <entry name="foo.png" serializer="svg2png">
//              <svg>...</svg>
//            </entry>

public class ZipArchiveSerializer extends AbstractSerializer
{
    /**
     * The namespace for elements handled by this serializer, 
     * "http://apache.org/cocoon/zip-archive/1.0".
     */
    public static final String ZIP_NAMESPACE = "http://apache.org/cocoon/zip-archive/1.0";
    
    /** The Zip stream where entries will be written */
    protected ZipOutputStream zipOutput;
    
    /** Have we encountered the toplevel "zip" element ? */
    protected boolean inZip = false;
    
    /** The resolver to get sources */
    protected SourceResolver resolver;
    
    /** Temporary byte buffer to read source data */
    protected byte[] buffer = new byte[1024];
    
    /**
     * Always return "application/x-zip" which is the default for Zip archives.
     */
    public String getMimeType()
    {
        return "application/x-zip";
    }

    /**
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException
    {
        this.zipOutput = new ZipOutputStream(this.output);
        this.inZip = false;
        this.resolver = CocoonComponentManager.getCurrentEnvironment();
    }

    /**
     * @see org.xml.sax.ContentHandler#startElement(String, String, String, Attributes)
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException
    {
        if (!inZip) {
            // expecting "zip" as the first element
            if (namespaceURI.equals(ZIP_NAMESPACE) && localName.equals("archive")) {
                this.inZip = true;
            } else {
                throw new SAXException("Expecting 'archive' root element (got '" + localName + "')");
            }
        } else {
            // expecting "entry" element
            if (namespaceURI.equals(ZIP_NAMESPACE) && localName.equals("entry")) {
                // Get the source
                addEntry(atts);
            } else {
                throw new SAXException("Expecting 'entry' element (got '" + localName + "')");
            }
        }
    }

    /**
     * Add an entry in the archive.
     * @param atts the attributes that describe the entry
     */
    protected void addEntry(Attributes atts) throws SAXException
    {
        String name = atts.getValue("name");
        if (name == null) {
            throw new SAXException("No name given to the Zip entry");
        }
        
        String src = atts.getValue("src");
        if (src == null) {
            throw new SAXException("No source given for the Zip entry");
        }
        
        try {
            // Create a new Zip entry
            ZipEntry entry = new ZipEntry(name);
            this.zipOutput.putNextEntry(entry);
            
            // Get the source and its data
            Source source = resolver.resolve(src);
            InputStream sourceInput = source.getInputStream();
            
            // Copy the source to the zip
            int len;
            while ((len = sourceInput.read(this.buffer)) > 0) {
                this.zipOutput.write(this.buffer, 0, len);
            }
            
            // and close the entry
            this.zipOutput.closeEntry();
            
        } catch(RuntimeException re) {
            throw re;
        } catch(SAXException se) {
            throw se;
        } catch(Exception e) {
            throw new SAXException(e);
        }
    }

    /**
     * @see org.xml.sax.ContentHandler#endDocument()
     */
    public void endDocument() throws SAXException
    {
        try {
            // Close the zip archive
            this.zipOutput.finish();
            
        } catch(IOException ioe) {
            throw new SAXException(ioe);
        }
    }
}
